@@ -1,14 +1,20 @@
-{-# LANGUAGE BangPatterns, CPP, MagicHash,
-             ScopedTypeVariables, UnliftedFFITypes, DeriveDataTypeable,
-             DefaultSignatures, FlexibleContexts, TypeFamilies,
-             MultiParamTypeClasses, CApiFFI #-}
-
-{-# LANGUAGE Trustworthy #-}
-
-#if __GLASGOW_HASKELL__ >= 801
-{-# LANGUAGE PolyKinds #-} -- For TypeRep instances
-#endif
-
+{-# LANGUAGE BangPatterns               #-}
+{-# LANGUAGE CApiFFI                    #-}
+{-# LANGUAGE CPP                        #-}
+{-# LANGUAGE DefaultSignatures          #-}
+{-# LANGUAGE DerivingStrategies         #-}
+{-# LANGUAGE FlexibleContexts           #-}
+{-# LANGUAGE GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE MagicHash                  #-}
+{-# LANGUAGE MultiParamTypeClasses      #-}
+{-# LANGUAGE PackageImports             #-}
+{-# LANGUAGE PolyKinds                  #-}
+{-# LANGUAGE QuantifiedConstraints      #-}
+{-# LANGUAGE ScopedTypeVariables        #-}
+{-# LANGUAGE StandaloneDeriving         #-}
+{-# LANGUAGE Trustworthy                #-}
+{-# LANGUAGE TypeFamilies               #-}
+{-# LANGUAGE UnliftedFFITypes           #-}
 {-# OPTIONS_GHC -fno-warn-deprecations #-}
 
 ------------------------------------------------------------------------
@@ -49,6 +55,7 @@ module Data.Hashable.Class
     , hashByteArray
     , hashByteArrayWithSalt
     , defaultHashWithSalt
+    , defaultHash
       -- * Higher Rank Functions
     , hashWithSalt1
     , hashWithSalt2
@@ -62,176 +69,110 @@ module Data.Hashable.Class
     , traverseHashed
     ) where
 
-import Control.Applicative (Const(..))
-import Control.Exception (assert)
-import Control.DeepSeq (NFData(rnf))
-import qualified Data.ByteString as B
-import qualified Data.ByteString.Lazy as BL
-import qualified Data.ByteString.Unsafe as B
-import Data.Complex (Complex(..))
-import Data.Int (Int8, Int16, Int32, Int64)
-import Data.List (foldl')
-import Data.Ratio (Ratio, denominator, numerator)
-import qualified Data.Text as T
-import qualified Data.Text.Array as TA
-import qualified Data.Text.Internal as T
-import qualified Data.Text.Lazy as TL
-import Data.Version (Version(..))
-import Data.Word (Word8, Word16, Word32, Word64)
-import Foreign.Marshal.Utils (with)
-import Foreign.Ptr (Ptr, FunPtr, IntPtr, WordPtr, castPtr, castFunPtrToPtr, ptrToIntPtr)
-import Foreign.Storable (alignment, peek, sizeOf)
-import GHC.Base (ByteArray#)
-import GHC.Conc (ThreadId(..))
-import GHC.Prim (ThreadId#)
-import System.IO.Unsafe (unsafeDupablePerformIO)
-import System.Mem.StableName
-import Data.Unique (Unique, hashUnique)
-import qualified Data.IntMap as IntMap
-import qualified Data.IntSet as IntSet
-import qualified Data.Map as Map
-import qualified Data.Sequence as Seq
-import qualified Data.Set as Set
-import qualified Data.Tree as Tree
+import Control.Applicative    (Const (..))
+import Control.DeepSeq        (NFData (rnf))
+import Control.Exception      (assert)
+import Control.Monad.ST       (runST)
+import Data.Array.Byte        (ByteArray (..))
+import Data.Complex           (Complex (..))
+import Data.Fixed             (Fixed (..))
+import Data.Functor.Classes   (Eq1 (..), Eq2 (..), Ord1 (..), Show1 (..))
+import Data.Functor.Compose   (Compose (..))
+import Data.Functor.Identity  (Identity (..))
+import Data.Int               (Int16, Int8)
+import Data.Kind              (Type)
+import Data.List              (foldl')
+import Data.Proxy             (Proxy)
+import Data.Ratio             (Ratio, denominator, numerator)
+import Data.String            (IsString (..))
+import Data.Tuple             (Solo (..))
+import Data.Unique            (Unique, hashUnique)
+import Data.Version           (Version (..))
+import Data.Void              (Void, absurd)
+import Data.Word              (Word16, Word8)
+import Foreign.Ptr            (FunPtr, IntPtr, Ptr, WordPtr, castFunPtrToPtr, ptrToIntPtr)
+import Foreign.Storable       (alignment, sizeOf)
+import GHC.Base               (ByteArray#)
+import GHC.Conc               (ThreadId (..))
+import GHC.Fingerprint.Type   (Fingerprint (..))
+import GHC.Word               (Word (..))
+import System.Mem.StableName  (StableName, hashStableName)
+import Type.Reflection        (SomeTypeRep (..), TypeRep)
+import Type.Reflection.Unsafe (typeRepFingerprint)
 
--- As we use qualified F.Foldable, we don't get warnings with newer base
-import qualified Data.Foldable as F
-
-#if MIN_VERSION_base(4,7,0)
-import Data.Proxy (Proxy)
-#endif
-
-#if MIN_VERSION_base(4,7,0)
-import Data.Fixed (Fixed(..))
-#else
-import Data.Fixed (Fixed)
-import Unsafe.Coerce (unsafeCoerce)
-#endif
-
-#if MIN_VERSION_base(4,8,0)
-import Data.Functor.Identity (Identity(..))
-#endif
+import qualified Data.Array.Byte                as AB
+import qualified Data.ByteString                as B
+import qualified Data.ByteString.Lazy           as BL
+import qualified Data.ByteString.Short.Internal as BSI
+import qualified Data.Functor.Product           as FP
+import qualified Data.Functor.Sum               as FS
+import qualified Data.IntMap                    as IntMap
+import qualified Data.IntSet                    as IntSet
+import qualified Data.List.NonEmpty             as NE
+import qualified Data.Map                       as Map
+import qualified Data.Semigroup                 as Semi
+import qualified Data.Sequence                  as Seq
+import qualified Data.Set                       as Set
+import qualified Data.Text                      as T
+import qualified Data.Text.Array                as TA
+import qualified Data.Text.Internal             as T
+import qualified Data.Text.Lazy                 as TL
+import qualified Data.Tree                      as Tree
 
 import GHC.Generics
 
-#if   MIN_VERSION_base(4,10,0)
-import Type.Reflection (Typeable, TypeRep, SomeTypeRep(..))
-import Type.Reflection.Unsafe (typeRepFingerprint)
-import GHC.Fingerprint.Type(Fingerprint(..))
-#elif MIN_VERSION_base(4,8,0)
-import Data.Typeable (typeRepFingerprint, Typeable, TypeRep)
-import GHC.Fingerprint.Type(Fingerprint(..))
+#if MIN_VERSION_base(4,19,0)
+import GHC.Conc.Sync (fromThreadId)
 #else
-import Data.Typeable.Internal (Typeable, TypeRep (..))
-import GHC.Fingerprint.Type(Fingerprint(..))
+import GHC.Prim (ThreadId#)
+#if __GLASGOW_HASKELL__ >= 904
+import Foreign.C.Types (CULLong (..))
+#elif __GLASGOW_HASKELL__ >= 900
+import Foreign.C.Types (CLong (..))
+#else
+import Foreign.C.Types (CInt (..))
+#endif
 #endif
 
-import Foreign.C.Types (CInt(..))
-
-#if !(MIN_VERSION_base(4,8,0))
-import Data.Word (Word)
-#endif
-
-#if !(MIN_VERSION_bytestring(0,10,0))
-import qualified Data.ByteString.Lazy.Internal as BL  -- foldlChunks
-#endif
-
-#if MIN_VERSION_bytestring(0,10,4)
-import qualified Data.ByteString.Short.Internal as BSI
-#endif
-
-#ifdef VERSION_ghc_bignum
-import GHC.Num.BigNat (BigNat (..))
+import GHC.Exts        (Int (..), sizeofByteArray#)
+import GHC.Num.BigNat  (BigNat (..))
 import GHC.Num.Integer (Integer (..))
 import GHC.Num.Natural (Natural (..))
-import GHC.Exts (Int (..), sizeofByteArray#)
-#endif
 
-#ifdef VERSION_integer_gmp
 
-# if MIN_VERSION_integer_gmp(1,0,0)
-#  define MIN_VERSION_integer_gmp_1_0_0
-# endif
+import GHC.Float (castDoubleToWord64, castFloatToWord32)
 
-import GHC.Exts (Int(..))
-import GHC.Integer.GMP.Internals (Integer(..))
-# if defined(MIN_VERSION_integer_gmp_1_0_0)
-import GHC.Exts (sizeofByteArray#)
-import GHC.Integer.GMP.Internals (BigNat(BN#))
-# endif
-#endif
-
-#if MIN_VERSION_base(4,8,0)
-import Data.Void (Void, absurd)
-import GHC.Exts (Word(..))
-#ifndef VERSION_ghc_bignum
-import GHC.Natural (Natural(..))
-#endif
-#endif
-
-import Data.Functor.Classes (Eq1(..),Ord1(..),Show1(..))
-
--- Whether we have lifted classes, in particular, Eq2
-#ifndef MIN_VERSION_transformers
-#define LIFTED_FUNCTOR_CLASSES 1
+-- filepath >=1.4.100 && <1.5 has System.OsString.Internal.Types module
+#if MIN_VERSION_filepath(1,4,100) && !(MIN_VERSION_filepath(1,5,0))
+#define HAS_OS_STRING_filepath 1
 #else
-#if !(MIN_VERSION_transformers(0,4,0) && !MIN_VERSION_transformers(0,5,0))
-#define LIFTED_FUNCTOR_CLASSES 1
-#endif
+#define HAS_OS_STRING_filepath 0
 #endif
 
-#ifdef LIFTED_FUNCTOR_CLASSES
-import Data.Functor.Classes (Eq2)
-#endif
-
-#if MIN_VERSION_base(4,9,0)
-import qualified Data.List.NonEmpty as NE
-import Data.Semigroup
-
-import Data.Functor.Compose (Compose(..))
-import qualified Data.Functor.Product as FP
-import qualified Data.Functor.Sum as FS
-#endif
-
-#if MIN_VERSION_base(4,16,0)
-import Data.Tuple (Solo (..))
-#elif MIN_VERSION_base(4,15,0)
-import GHC.Tuple (Solo (..))
-#endif
-
-import Data.String (IsString(..))
-
-#if MIN_VERSION_base(4,9,0)
-import Data.Kind (Type)
+-- if we depend on os_string module, then it has System.OsString.Internal.Types
+-- module as well
+#ifdef MIN_VERSION_os_string
+#define HAS_OS_STRING_os_string 1
 #else
-#define Type *
+#define HAS_OS_STRING_os_string 0
+#endif
+
+#if HAS_OS_STRING_filepath && HAS_OS_STRING_os_string
+import "os-string" System.OsString.Internal.Types (OsString (..), PosixString (..), WindowsString (..))
+import qualified "filepath" System.OsString.Internal.Types as FP (OsString (..), PosixString (..), WindowsString (..))
+#elif HAS_OS_STRING_filepath || HAS_OS_STRING_os_string
+import System.OsString.Internal.Types (OsString (..), PosixString (..), WindowsString (..))
 #endif
 
 import Data.Hashable.Imports
 import Data.Hashable.LowLevel
-
-#ifdef VERSION_base_orphans
-import Data.Orphans ()
-#endif
-
-#ifdef VERSION_transformers_compat
-import Control.Monad.Trans.Instances ()
-#endif
-
-#ifdef VERSION_ghc_bignum_orphans
-import GHC.Num.Orphans ()
-#endif
-
-#ifdef VERSION_functor_classes_compat
-import Data.Map.Functor.Classes ()
-import Data.Set.Functor.Classes ()
-import Data.IntMap.Functor.Classes ()
-import Data.Sequence.Functor.Classes ()
-import Data.Tree.Functor.Classes ()
-#endif
+import Data.Hashable.XXH3
 
 #include "MachDeps.h"
 
+-- sometimes we need dependency of filepath, sometimes we dont
+import System.FilePath ()
+
 infixl 0 `hashWithSalt`
 
 ------------------------------------------------------------------------
@@ -241,9 +182,15 @@ infixl 0 `hashWithSalt`
 --
 -- Minimal implementation: 'hashWithSalt'.
 --
--- /Note:/ the hash is not guaranteed to be stable across
--- library versions, operating systems or architectures.
--- For stable hashing use named hashes: SHA256, CRC32 etc.
+-- 'Hashable' is intended exclusively for use in in-memory data structures.
+-- .
+-- 'Hashable' does /not/ have a fixed standard.
+-- This allows it to improve over time.
+-- .
+-- Because it does not have a fixed standard, different computers or computers on different versions of the code will observe different hash values.
+-- As such, 'Hashable' is not recommended for use other than in-memory datastructures.
+-- Specifically, 'Hashable' is not intended for network use or in applications which persist hashed values.
+-- For stable hashing use named hashes: sha256, crc32, xxhash etc.
 --
 -- If you are looking for 'Hashable' instance in @time@ package,
 -- check [time-compat](https://hackage.haskell.org/package/time-compat)
@@ -281,7 +228,7 @@ class Eq a => Hashable a where
     -- Instances might want to implement this method to provide a more
     -- efficient implementation than the default implementation.
     hash :: a -> Int
-    hash = hashWithSalt defaultSalt
+    hash = defaultHash
 
     default hashWithSalt :: (Generic a, GHashable Zero (Rep a)) => Int -> a -> Int
     hashWithSalt = genericHashWithSalt
@@ -305,7 +252,7 @@ newtype instance HashArgs One  a = HashArgs1 (Int -> a -> Int)
 class GHashable arity f where
     ghashWithSalt :: HashArgs arity a -> Int -> f a -> Int
 
-class Eq1 t => Hashable1 t where
+class (Eq1 t, forall a. Hashable a => Hashable (t a)) => Hashable1 t where
     -- | Lift a hashing function through the type constructor.
     liftHashWithSalt :: (Int -> a -> Int) -> Int -> t a -> Int
 
@@ -320,11 +267,7 @@ genericLiftHashWithSalt :: (Generic1 t, GHashable One (Rep1 t)) => (Int -> a ->
 genericLiftHashWithSalt = \h salt -> ghashWithSalt (HashArgs1 h) salt . from1
 {-# INLINE genericLiftHashWithSalt #-}
 
-#if LIFTED_FUNCTOR_CLASSES
-class Eq2 t => Hashable2 t where
-#else
-class Hashable2 t where
-#endif
+class (Eq2 t, forall a. Hashable a => Hashable1 (t a)) => Hashable2 t where
     -- | Lift a hashing function through the binary type constructor.
     liftHashWithSalt2 :: (Int -> a -> Int) -> (Int -> b -> Int) -> Int -> t a b -> Int
 
@@ -350,9 +293,19 @@ defaultLiftHashWithSalt h = liftHashWithSalt2 hashWithSalt h
 -- cannot also provide a default implementation for that method for
 -- the non-generic instance use case. Instead we provide
 -- 'defaultHashWith'.
+--
+-- @since 1.4.3.0
+--
 defaultHashWithSalt :: Hashable a => Int -> a -> Int
 defaultHashWithSalt salt x = salt `hashInt` hash x
 
+-- | Default implementation of 'hash' based on 'hashWithSalt'.
+--
+-- @since 1.4.3.0
+--
+defaultHash :: Hashable a => a -> Int
+defaultHash = hashWithSalt defaultSalt
+
 -- | Transform a value into a 'Hashable' value, then hash the
 -- transformed value using the given salt.
 --
@@ -412,6 +365,7 @@ instance Hashable Word32 where
     hashWithSalt = defaultHashWithSalt
 
 instance Hashable Word64 where
+    hash = fromIntegral
     hashWithSalt = hashWord64
 
 instance Hashable () where
@@ -430,40 +384,17 @@ instance Hashable Char where
     hash = fromEnum
     hashWithSalt = defaultHashWithSalt
 
-#if defined(MIN_VERSION_integer_gmp_1_0_0) || defined(VERSION_ghc_bignum)
 instance Hashable BigNat where
-    hashWithSalt salt (BN# ba) = hashByteArrayWithSalt ba 0 numBytes salt
-                                 `hashWithSalt` size
-      where
-        size     = numBytes `quot` SIZEOF_HSWORD
-        numBytes = I# (sizeofByteArray# ba)
-#endif
+    hashWithSalt salt (BN# ba) = hashWithSalt salt (ByteArray ba)
 
-#if MIN_VERSION_base(4,8,0)
 instance Hashable Natural where
-# if defined(VERSION_ghc_bignum)
     hash (NS n)   = hash (W# n)
     hash (NB bn)  = hash (BN# bn)
 
     hashWithSalt salt (NS n)  = hashWithSalt salt (W# n)
     hashWithSalt salt (NB bn) = hashWithSalt salt (BN# bn)
-# else
-# if defined(MIN_VERSION_integer_gmp_1_0_0)
-    hash (NatS# n)   = hash (W# n)
-    hash (NatJ# bn)  = hash bn
-
-    hashWithSalt salt (NatS# n)   = hashWithSalt salt (W# n)
-    hashWithSalt salt (NatJ# bn)  = hashWithSalt salt bn
-# else
-    hash (Natural n) = hash n
-
-    hashWithSalt salt (Natural n) = hashWithSalt salt n
-# endif
-# endif
-#endif
 
 instance Hashable Integer where
-#if defined(VERSION_ghc_bignum)
     hash (IS n)  = I# n
     hash (IP bn) = hash (BN# bn)
     hash (IN bn) = negate (hash (BN# bn))
@@ -471,46 +402,6 @@ instance Hashable Integer where
     hashWithSalt salt (IS n)  = hashWithSalt salt (I# n)
     hashWithSalt salt (IP bn) = hashWithSalt salt (BN# bn)
     hashWithSalt salt (IN bn) = negate (hashWithSalt salt (BN# bn))
-#else
-#if defined(VERSION_integer_gmp)
-# if defined(MIN_VERSION_integer_gmp_1_0_0)
-    hash (S# n)   = (I# n)
-    hash (Jp# bn) = hash bn
-    hash (Jn# bn) = negate (hash bn)
-
-    hashWithSalt salt (S# n)   = hashWithSalt salt (I# n)
-    hashWithSalt salt (Jp# bn) = hashWithSalt salt bn
-    hashWithSalt salt (Jn# bn) = negate (hashWithSalt salt bn)
-# else
-    hash (S# int) = I# int
-    hash n@(J# size# byteArray)
-        | n >= minInt && n <= maxInt = fromInteger n :: Int
-        | otherwise = let size = I# size#
-                          numBytes = SIZEOF_HSWORD * abs size
-                      in hashByteArrayWithSalt byteArray 0 numBytes defaultSalt
-                         `hashWithSalt` size
-      where minInt = fromIntegral (minBound :: Int)
-            maxInt = fromIntegral (maxBound :: Int)
-
-    hashWithSalt salt (S# n) = hashWithSalt salt (I# n)
-    hashWithSalt salt n@(J# size# byteArray)
-        | n >= minInt && n <= maxInt = hashWithSalt salt (fromInteger n :: Int)
-        | otherwise = let size = I# size#
-                          numBytes = SIZEOF_HSWORD * abs size
-                      in hashByteArrayWithSalt byteArray 0 numBytes salt
-                         `hashWithSalt` size
-      where minInt = fromIntegral (minBound :: Int)
-            maxInt = fromIntegral (maxBound :: Int)
-# endif
-#else
-    hashWithSalt salt = foldl' hashWithSalt salt . go
-      where
-        go n | inBounds n = [fromIntegral n :: Int]
-             | otherwise   = fromIntegral n : go (n `shiftR` WORD_SIZE_IN_BITS)
-        maxInt = fromIntegral (maxBound :: Int)
-        inBounds x = x >= fromIntegral (minBound :: Int) && x <= maxInt
-#endif
-#endif
 
 instance Hashable a => Hashable (Complex a) where
     {-# SPECIALIZE instance Hashable (Complex Double) #-}
@@ -520,12 +411,7 @@ instance Hashable a => Hashable (Complex a) where
 instance Hashable1 Complex where
     liftHashWithSalt h s (r :+ i) = s `h` r `h` i
 
-#if MIN_VERSION_base(4,9,0)
--- Starting with base-4.9, numerator/denominator don't need 'Integral' anymore
 instance Hashable a => Hashable (Ratio a) where
-#else
-instance (Integral a, Hashable a) => Hashable (Ratio a) where
-#endif
     {-# SPECIALIZE instance Hashable (Ratio Integer) #-}
     hash a = hash (numerator a) `hashWithSalt` denominator a
     hashWithSalt s a = s `hashWithSalt` numerator a `hashWithSalt` denominator a
@@ -541,7 +427,7 @@ instance Hashable Float where
         | isIEEE x =
             assert (sizeOf x >= sizeOf (0::Word32) &&
                     alignment x >= alignment (0::Word32)) $
-            hash ((unsafeDupablePerformIO $ with x $ peek . castPtr) :: Word32)
+            hash (castFloatToWord32 x)
         | otherwise = hash (show x)
     hashWithSalt = defaultHashWithSalt
 
@@ -556,7 +442,7 @@ instance Hashable Double where
         | isIEEE x =
             assert (sizeOf x >= sizeOf (0::Word64) &&
                     alignment x >= alignment (0::Word64)) $
-            hash ((unsafeDupablePerformIO $ with x $ peek . castPtr) :: Word64)
+            hash (castDoubleToWord64 x)
         | otherwise = hash (show x)
     hashWithSalt = defaultHashWithSalt
 
@@ -589,7 +475,6 @@ instance Hashable2 Either where
     liftHashWithSalt2 _ h s (Right b) = s `hashInt` distinguisher `h` b
 
 instance (Hashable a1, Hashable a2) => Hashable (a1, a2) where
-    hash (a1, a2) = hash a1 `hashWithSalt` a2
     hashWithSalt = hashWithSalt1
 
 instance Hashable a1 => Hashable1 ((,) a1) where
@@ -599,7 +484,6 @@ instance Hashable2 (,) where
     liftHashWithSalt2 h1 h2 s (a1, a2) = s `h1` a1 `h2` a2
 
 instance (Hashable a1, Hashable a2, Hashable a3) => Hashable (a1, a2, a3) where
-    hash (a1, a2, a3) = hash a1 `hashWithSalt` a2 `hashWithSalt` a3
     hashWithSalt = hashWithSalt1
 
 instance (Hashable a1, Hashable a2) => Hashable1 ((,,) a1 a2) where
@@ -611,8 +495,6 @@ instance Hashable a1 => Hashable2 ((,,) a1) where
 
 instance (Hashable a1, Hashable a2, Hashable a3, Hashable a4) =>
          Hashable (a1, a2, a3, a4) where
-    hash (a1, a2, a3, a4) = hash a1 `hashWithSalt` a2
-                            `hashWithSalt` a3 `hashWithSalt` a4
     hashWithSalt = hashWithSalt1
 
 instance (Hashable a1, Hashable a2, Hashable a3) => Hashable1 ((,,,) a1 a2 a3) where
@@ -624,9 +506,6 @@ instance (Hashable a1, Hashable a2) => Hashable2 ((,,,) a1 a2) where
 
 instance (Hashable a1, Hashable a2, Hashable a3, Hashable a4, Hashable a5)
       => Hashable (a1, a2, a3, a4, a5) where
-    hash (a1, a2, a3, a4, a5) =
-        hash a1 `hashWithSalt` a2 `hashWithSalt` a3
-        `hashWithSalt` a4 `hashWithSalt` a5
     hashWithSalt s (a1, a2, a3, a4, a5) =
         s `hashWithSalt` a1 `hashWithSalt` a2 `hashWithSalt` a3
         `hashWithSalt` a4 `hashWithSalt` a5
@@ -645,9 +524,6 @@ instance (Hashable a1, Hashable a2, Hashable a3)
 
 instance (Hashable a1, Hashable a2, Hashable a3, Hashable a4, Hashable a5,
           Hashable a6) => Hashable (a1, a2, a3, a4, a5, a6) where
-    hash (a1, a2, a3, a4, a5, a6) =
-        hash a1 `hashWithSalt` a2 `hashWithSalt` a3
-        `hashWithSalt` a4 `hashWithSalt` a5 `hashWithSalt` a6
     hashWithSalt s (a1, a2, a3, a4, a5, a6) =
         s `hashWithSalt` a1 `hashWithSalt` a2 `hashWithSalt` a3
         `hashWithSalt` a4 `hashWithSalt` a5 `hashWithSalt` a6
@@ -667,9 +543,6 @@ instance (Hashable a1, Hashable a2, Hashable a3,
 instance (Hashable a1, Hashable a2, Hashable a3, Hashable a4, Hashable a5,
           Hashable a6, Hashable a7) =>
          Hashable (a1, a2, a3, a4, a5, a6, a7) where
-    hash (a1, a2, a3, a4, a5, a6, a7) =
-        hash a1 `hashWithSalt` a2 `hashWithSalt` a3
-        `hashWithSalt` a4 `hashWithSalt` a5 `hashWithSalt` a6 `hashWithSalt` a7
     hashWithSalt s (a1, a2, a3, a4, a5, a6, a7) =
         s `hashWithSalt` a1 `hashWithSalt` a2 `hashWithSalt` a3
         `hashWithSalt` a4 `hashWithSalt` a5 `hashWithSalt` a6 `hashWithSalt` a7
@@ -689,7 +562,7 @@ instance Hashable (StableName a) where
     hash = hashStableName
     hashWithSalt = defaultHashWithSalt
 
--- Auxillary type for Hashable [a] definition
+-- Auxiliary type for Hashable [a] definition
 data SPInt = SP !Int !Int
 
 instance Hashable a => Hashable [a] where
@@ -703,66 +576,110 @@ instance Hashable1 [] where
         step (SP s l) x   = SP (h s x) (l + 1)
 
 instance Hashable B.ByteString where
-    hashWithSalt salt bs = unsafeDupablePerformIO $
-                           B.unsafeUseAsCStringLen bs $ \(p, len) ->
-                           hashPtrWithSalt p (fromIntegral len) (hashWithSalt salt len)
+    hash bs = fromIntegral (xxh3_64bit_withSeed_bs bs 0)
+
+    hashWithSalt salt bs =
+        fromIntegral (xxh3_64bit_withSeed_bs bs (fromIntegral (hashWithSalt salt len)))
+      where
+        len = B.length bs
 
 instance Hashable BL.ByteString where
-    hashWithSalt salt = finalise . BL.foldlChunks step (SP salt 0)
+    hashWithSalt salt lbs = runST $ do
+        s <- xxh3_64bit_createState
+        xxh3_64bit_reset_withSeed s (fromIntegral salt)
+        len <- BL.foldrChunks (step s) return lbs 0
+        xxh3_64bit_update_w64 s len
+        digest <- xxh3_64bit_digest s
+        return (fromIntegral digest)
       where
-        finalise (SP s l) = hashWithSalt s l
-        step (SP s l) bs  = unsafeDupablePerformIO $
-                            B.unsafeUseAsCStringLen bs $ \(p, len) -> do
-                                s' <- hashPtrWithSalt p (fromIntegral len) s
-                                return (SP s' (l + len))
+        step s bs next !acc = do
+            xxh3_64bit_update_bs s bs
+            next (acc + fromIntegral (B.length bs))
 
-#if MIN_VERSION_bytestring(0,10,4)
 instance Hashable BSI.ShortByteString where
-    hashWithSalt salt sbs@(BSI.SBS ba) =
-        hashByteArrayWithSalt ba 0 (BSI.length sbs) (hashWithSalt salt (BSI.length sbs))
+    hash (BSI.SBS ba) = hash (ByteArray ba)
+    hashWithSalt salt (BSI.SBS ba) = hashWithSalt salt (ByteArray ba)
+
+#if HAS_OS_STRING_filepath || HAS_OS_STRING_os_string
+-- | @since 1.4.2.0
+deriving newtype instance Hashable PosixString
+
+-- | @since 1.4.2.0
+deriving newtype instance Hashable WindowsString
+
+-- | @since 1.4.2.0
+deriving newtype instance Hashable OsString
+#endif
+
+#if HAS_OS_STRING_filepath && HAS_OS_STRING_os_string
+deriving newtype instance Hashable FP.PosixString
+deriving newtype instance Hashable FP.WindowsString
+deriving newtype instance Hashable FP.OsString
 #endif
 
 #if MIN_VERSION_text(2,0,0)
 
 instance Hashable T.Text where
+    hash (T.Text (TA.ByteArray arr) off len) =
+        fromIntegral (xxh3_64bit_withSeed_ba (ByteArray arr) off len 0)
     hashWithSalt salt (T.Text (TA.ByteArray arr) off len) =
-        hashByteArrayWithSalt arr off len (hashWithSalt salt len)
+        fromIntegral (xxh3_64bit_withSeed_ba (ByteArray arr) off len (fromIntegral (hashWithSalt salt len)))
 
 instance Hashable TL.Text where
-    hashWithSalt salt = finalise . TL.foldlChunks step (SP salt 0)
+    hashWithSalt salt lt = runST $ do
+        s <- xxh3_64bit_createState
+        xxh3_64bit_reset_withSeed s (fromIntegral salt)
+        len <- TL.foldrChunks (step s) return lt 0
+        xxh3_64bit_update_w64 s len
+        digest <- xxh3_64bit_digest s
+        return (fromIntegral digest)
       where
-        finalise (SP s l) = hashWithSalt s l
-        step (SP s l) (T.Text (TA.ByteArray arr) off len) = SP
-            (hashByteArrayWithSalt arr off len s)
-            (l + len)
+        step s (T.Text (TA.ByteArray arr) off len) next !acc = do
+            xxh3_64bit_update_ba s (ByteArray arr) off len
+            next (acc + fromIntegral len)
 
 #else
 
 instance Hashable T.Text where
+    hash (T.Text arr off len) =
+        fromIntegral (xxh3_64bit_withSeed_ba (ByteArray (TA.aBA arr)) (unsafeShiftL off 1) (unsafeShiftL len 1) 0)
     hashWithSalt salt (T.Text arr off len) =
-        hashByteArrayWithSalt (TA.aBA arr) (off `shiftL` 1) (len `shiftL` 1)
-        (hashWithSalt salt len)
+        fromIntegral (xxh3_64bit_withSeed_ba (ByteArray (TA.aBA arr)) (unsafeShiftL off 1) (unsafeShiftL len 1) (fromIntegral (hashWithSalt salt len)))
 
 instance Hashable TL.Text where
-    hashWithSalt salt = finalise . TL.foldlChunks step (SP salt 0)
+    hashWithSalt salt lt = runST $ do
+        s <- xxh3_64bit_createState
+        xxh3_64bit_reset_withSeed s (fromIntegral salt)
+        len <- TL.foldrChunks (step s) return lt 0
+        xxh3_64bit_update_w64 s len
+        digest <- xxh3_64bit_digest s
+        return (fromIntegral digest)
       where
-        finalise (SP s l) = hashWithSalt s l
-        step (SP s l) (T.Text arr off len) = SP
-            (hashByteArrayWithSalt (TA.aBA arr) (off `shiftL` 1) (len `shiftL` 1) s)
-            (l + len)
+        step s (T.Text arr off len) next !acc = do
+            xxh3_64bit_update_ba s (ByteArray (TA.aBA arr)) (unsafeShiftL off 1) (unsafeShiftL len 1)
+            next (acc + fromIntegral len)
 
 #endif
 
--- | Compute the hash of a ThreadId.
-hashThreadId :: ThreadId -> Int
-hashThreadId (ThreadId t) = hash (fromIntegral (getThreadId t) :: Int)
+#if !MIN_VERSION_base(4,19,0)
+fromThreadId :: ThreadId -> Word64
+fromThreadId (ThreadId t) = fromIntegral (getThreadId t)
 
 -- this cannot be capi, as GHC panics.
 foreign import ccall unsafe "rts_getThreadId" getThreadId
+#if __GLASGOW_HASKELL__ >= 904
+    -- https://gitlab.haskell.org/ghc/ghc/-/merge_requests/6163
+    :: ThreadId# -> CULLong
+#elif __GLASGOW_HASKELL__ >= 900
+    -- https://gitlab.haskell.org/ghc/ghc/-/merge_requests/1254
+    :: ThreadId# -> CLong
+#else
     :: ThreadId# -> CInt
+#endif
+#endif
 
 instance Hashable ThreadId where
-    hash = hashThreadId
+    hash = hash . fromThreadId
     hashWithSalt = defaultHashWithSalt
 
 instance Hashable (Ptr a) where
@@ -788,8 +705,6 @@ instance Hashable Fingerprint where
     hashWithSalt = defaultHashWithSalt
     {-# INLINE hash #-}
 
-#if MIN_VERSION_base(4,10,0)
-
 hashTypeRep :: Type.Reflection.TypeRep a -> Int
 hashTypeRep tr =
     let Fingerprint x _ = typeRepFingerprint tr in fromIntegral x
@@ -804,32 +719,10 @@ instance Hashable (Type.Reflection.TypeRep a) where
     hashWithSalt = defaultHashWithSalt
     {-# INLINE hash #-}
 
-#else
-
--- | Compute the hash of a TypeRep, in various GHC versions we can do this quickly.
-hashTypeRep :: TypeRep -> Int
-{-# INLINE hashTypeRep #-}
-#if   MIN_VERSION_base(4,8,0)
--- Fingerprint is just the MD5, so taking any Int from it is fine
-hashTypeRep tr = let Fingerprint x _ = typeRepFingerprint tr in fromIntegral x
-#else
--- Fingerprint is just the MD5, so taking any Int from it is fine
-hashTypeRep (TypeRep (Fingerprint x _) _ _) = fromIntegral x
-#endif
-
-instance Hashable TypeRep where
-    hash = hashTypeRep
-    hashWithSalt = defaultHashWithSalt
-    {-# INLINE hash #-}
-
-#endif
-
 ----------------------------------------------------------------------------
 
-#if MIN_VERSION_base(4,8,0)
 instance Hashable Void where
     hashWithSalt _ = absurd
-#endif
 
 -- | Compute a hash value for the content of this pointer.
 hashPtr :: Ptr a      -- ^ pointer to the data to hash
@@ -854,26 +747,14 @@ instance Hashable Version where
     hashWithSalt salt (Version branch tags) =
         salt `hashWithSalt` branch `hashWithSalt` tags
 
-#if MIN_VERSION_base(4,7,0)
-instance Hashable (Fixed a) where
-    hashWithSalt salt (MkFixed i) = hashWithSalt salt i
-#else
-instance Hashable (Fixed a) where
-    hashWithSalt salt x = hashWithSalt salt (unsafeCoerce x :: Integer)
-#endif
+deriving newtype instance Hashable (Fixed a)
 
-
-#if MIN_VERSION_base(4,8,0)
--- TODO: make available on all base
-instance Hashable a => Hashable (Identity a) where
-    hashWithSalt = hashWithSalt1
+deriving newtype instance Hashable a => Hashable (Identity a)
 instance Hashable1 Identity where
     liftHashWithSalt h salt (Identity x) = h salt x
-#endif
 
 -- Using hashWithSalt1 would cause needless constraint
-instance Hashable a => Hashable (Const a b) where
-    hashWithSalt salt (Const x) = hashWithSalt salt x
+deriving newtype instance Hashable a => Hashable (Const a b)
 
 instance Hashable a => Hashable1 (Const a) where
     liftHashWithSalt = defaultLiftHashWithSalt
@@ -881,17 +762,13 @@ instance Hashable a => Hashable1 (Const a) where
 instance Hashable2 Const where
     liftHashWithSalt2 f _ salt (Const x) = f salt x
 
-#if MIN_VERSION_base(4,7,0)
 instance Hashable (Proxy a) where
     hash _ = 0
     hashWithSalt s _ = s
 
 instance Hashable1 Proxy where
     liftHashWithSalt _ s _ = s
-#endif
 
--- instances formerly provided by 'semigroups' package
-#if MIN_VERSION_base(4,9,0)
 instance Hashable a => Hashable (NE.NonEmpty a) where
     hashWithSalt p (a NE.:| as) = p `hashWithSalt` a `hashWithSalt` as
 
@@ -899,34 +776,24 @@ instance Hashable a => Hashable (NE.NonEmpty a) where
 instance Hashable1 NE.NonEmpty where
     liftHashWithSalt h salt (a NE.:| as) = liftHashWithSalt h (h salt a) as
 
-instance Hashable a => Hashable (Min a) where
-    hashWithSalt p (Min a) = hashWithSalt p a
-
-instance Hashable a => Hashable (Max a) where
-    hashWithSalt p (Max a) = hashWithSalt p a
+deriving newtype instance Hashable a => Hashable (Semi.Min a)
+deriving newtype instance Hashable a => Hashable (Semi.Max a)
 
 -- | __Note__: Prior to @hashable-1.3.0.0@ the hash computation included the second argument of 'Arg' which wasn't consistent with its 'Eq' instance.
 --
+-- Since @hashable-1.5.0.0@, @hash (Semi.arg a _) = hash a@
+--
 -- @since 1.3.0.0
-instance Hashable a => Hashable (Arg a b) where
-    hashWithSalt p (Arg a _) = hashWithSalt p a
-
-instance Hashable a => Hashable (First a) where
-    hashWithSalt p (First a) = hashWithSalt p a
-
-
-instance Hashable a => Hashable (Last a) where
-    hashWithSalt p (Last a) = hashWithSalt p a
-
-
-instance Hashable a => Hashable (WrappedMonoid a) where
-    hashWithSalt p (WrapMonoid a) = hashWithSalt p a
+instance Hashable a => Hashable (Semi.Arg a b) where
+    hash (Semi.Arg a _) = hash a
+    hashWithSalt p (Semi.Arg a _) = hashWithSalt p a
 
+deriving newtype instance Hashable a => Hashable (Semi.First a)
+deriving newtype instance Hashable a => Hashable (Semi.Last a)
+deriving newtype instance Hashable a => Hashable (Semi.WrappedMonoid a)
 
 #if !MIN_VERSION_base(4,16,0)
-instance Hashable a => Hashable (Option a) where
-    hashWithSalt p (Option a) = hashWithSalt p a
-
+deriving newtype instance Hashable a => Hashable (Semi.Option a)
 #endif
 
 -- TODO: this instance is removed as there isn't Eq1 Min/Max, ...
@@ -951,16 +818,10 @@ instance Hashable a => Hashable (Option a) where
 -- | @since 1.3.1.0
 -- instance Hashable1 Option where liftHashWithSalt h salt (Option a) = liftHashWithSalt h salt a
 #endif
-#endif
 
--- instances for @Data.Functor.{Product,Sum,Compose}@, present
--- in base-4.9 and onward.
-#if MIN_VERSION_base(4,9,0)
-
--- | In general, @hash (Compose x) ≠ hash x@. However, @hashWithSalt@ satisfies
--- its variant of this equivalence.
-instance (Hashable1 f, Hashable1 g, Hashable a) => Hashable (Compose f g a) where
-    hashWithSalt = hashWithSalt1
+instance (Hashable (f (g a))) => Hashable (Compose f g a) where
+    hash (Compose x) = hash x
+    hashWithSalt p (Compose x) = hashWithSalt p x
 
 instance (Hashable1 f, Hashable1 g) => Hashable1 (Compose f g) where
     liftHashWithSalt h s = liftHashWithSalt (liftHashWithSalt h) s . getCompose
@@ -968,20 +829,38 @@ instance (Hashable1 f, Hashable1 g) => Hashable1 (Compose f g) where
 instance (Hashable1 f, Hashable1 g) => Hashable1 (FP.Product f g) where
     liftHashWithSalt h s (FP.Pair a b) = liftHashWithSalt h (liftHashWithSalt h s a) b
 
-instance (Hashable1 f, Hashable1 g, Hashable a) => Hashable (FP.Product f g a) where
-    hashWithSalt = hashWithSalt1
+instance (Hashable (f a), Hashable (g a)) => Hashable (FP.Product f g a) where
+    hashWithSalt s (FP.Pair a b) = s `hashWithSalt` a `hashWithSalt` b
 
 instance (Hashable1 f, Hashable1 g) => Hashable1 (FS.Sum f g) where
     liftHashWithSalt h s (FS.InL a) = liftHashWithSalt h (s `hashInt` 0) a
     liftHashWithSalt h s (FS.InR a) = liftHashWithSalt h (s `hashInt` distinguisher) a
 
-instance (Hashable1 f, Hashable1 g, Hashable a) => Hashable (FS.Sum f g a) where
-    hashWithSalt = hashWithSalt1
-#endif
+instance (Hashable (f a), Hashable (g a)) => Hashable (FS.Sum f g a) where
+    hashWithSalt s (FS.InL a) = hashWithSalt (s `hashInt` 0) a
+    hashWithSalt s (FS.InR a) = hashWithSalt (s `hashInt` distinguisher) a
+
+-- | This instance was available since 1.4.1.0 only for GHC-9.4+
+--
+-- @since 1.4.2.0
+--
+instance Hashable AB.ByteArray where
+    hash ba@(AB.ByteArray ba') =
+        fromIntegral (xxh3_64bit_withSeed_ba ba 0 len 0)
+      where
+        !len = I# (sizeofByteArray# ba')
+
+    hashWithSalt salt ba@(AB.ByteArray ba') =
+        fromIntegral (xxh3_64bit_withSeed_ba ba 0 len (fromIntegral (hashWithSalt salt len)))
+      where
+        !len = I# (sizeofByteArray# ba')
+
+-------------------------------------------------------------------------------
+-- Hashed
+-------------------------------------------------------------------------------
 
 -- | A hashable value along with the result of the 'hash' function.
 data Hashed a = Hashed a {-# UNPACK #-} !Int
-  deriving (Typeable)
 
 -- | Wrap a hashable value, caching the 'hash' function result.
 hashed :: Hashable a => a -> Hashed a
@@ -1022,7 +901,8 @@ instance Hashable1 Hashed where
 instance (IsString a, Hashable a) => IsString (Hashed a) where
   fromString s = let r = fromString s in Hashed r (hash r)
 
-instance F.Foldable Hashed where
+instance Foldable Hashed where
+  foldMap f (Hashed a _) = f a
   foldr f acc (Hashed a _) = f a acc
 
 instance NFData a => NFData (Hashed a) where
@@ -1036,17 +916,8 @@ mapHashed f (Hashed a _) = hashed (f a)
 traverseHashed :: (Hashable b, Functor f) => (a -> f b) -> Hashed a -> f (Hashed b)
 traverseHashed f (Hashed a _) = fmap hashed (f a)
 
-#if MIN_VERSION_base(4,9,0)
-#define LIFTED_FUNCTOR_CLASSES 1
-#elif defined(MIN_VERSION_transformers)
-#if !(MIN_VERSION_transformers(0,4,0)) || MIN_VERSION_transformers(0,5,0)
-#define LIFTED_FUNCTOR_CLASSES 1
-#endif
-#endif
-
 -- instances for @Data.Functor.Classes@ higher rank typeclasses
 -- in base-4.9 and onward.
-#if LIFTED_FUNCTOR_CLASSES
 instance Eq1 Hashed where
   liftEq f (Hashed a ha) (Hashed b hb) = ha == hb && f a b
 
@@ -1056,11 +927,6 @@ instance Ord1 Hashed where
 instance Show1 Hashed where
   liftShowsPrec sp _ d (Hashed a _) = showParen (d > 10) $
     showString "hashed " . sp 11 a
-#else
-instance Eq1 Hashed where eq1 = (==)
-instance Ord1 Hashed where compare1 = compare
-instance Show1 Hashed where showsPrec1 = showsPrec
-#endif
 
 -------------------------------------------------------------------------------
 -- containers
@@ -1111,7 +977,7 @@ instance Hashable IntSet.IntSet where
 
 -- | @since 1.3.4.0
 instance Hashable1 Seq.Seq where
-    liftHashWithSalt h s x = F.foldl' h (hashWithSalt s (Seq.length x)) x
+    liftHashWithSalt h s x = foldl' h (hashWithSalt s (Seq.length x)) x
 
 -- | @since 1.3.4.0
 instance Hashable v => Hashable (Seq.Seq v) where
@@ -1130,9 +996,7 @@ instance Hashable v => Hashable (Tree.Tree v) where
 -- Solo
 -------------------------------------------------------------------------------
 
-#if MIN_VERSION_base(4,15,0)
 instance Hashable a => Hashable (Solo a) where
     hashWithSalt = hashWithSalt1
 instance Hashable1 Solo where
     liftHashWithSalt h salt (Solo x) = h salt x
-#endif
